class ResourcePostStatus:
    SUCCESS = 'SUCCESS'
    FAILED = 'FAILED'


class NodeResources:
    INPUT = 'inputs'
    OUTPUT = 'outputs'
    CLOUD_INPUT = 'cloud_inputs'
    CLOUD_OUTPUT = 'cloud_outputs'
    PARAM = 'params'
    LOG = 'logs'
