class ParameterTypes:
    STR = 'str'
    INT = 'int'
    BOOL = 'bool'
    TEXT = 'text'
    ENUM = 'enum'
    LIST_STR = 'list_str'
    LIST_INT = 'list_int'
    LIST_NODE = 'list_node'
    CODE = 'code'
