class Collections:
    GROUPS = 'groups'
    NODE_CACHE = 'node_cache'
    RUN_CANCELLATIONS = 'run_cancellations'
    RUNS = 'runs'
    TEMPLATES = 'templates'
    USERS = 'users'
    WORKER_STATE = 'worker_state'
